
#pragma once

#include "darray.h"
#include "filterTypes.h"

class MovingAverageImpl
{
private:

public:
    MovingAverageImpl(
        darray & input, 
        darray & output,
        int maPeriod,
        FilterType maMethod)

        : Close(input), ExtMapBuffer(output),
        MA_Period(maPeriod),
        MA_Method(maMethod)
    {
        Bars = input.size();
        start();
    }

    int IndicatorCounted() {
        return 0;
    }

public:
    //---- indicator parameters
    int MA_Period;
    int MA_Shift;
    FilterType MA_Method;
    
    //---- indicator buffers
    darray & Close;
    darray & ExtMapBuffer;
    //----
    int ExtCountedBars;
    int Bars;

    //+------------------------------------------------------------------+
    //| Custom indicator initialization function                         |
    //+------------------------------------------------------------------+
    int init()
    {
        int    draw_begin;
        return(0);
    }

    //+------------------------------------------------------------------+
    //|                                                                  |
    //+------------------------------------------------------------------+
    int start()
    {
        if(Bars<=MA_Period) return(0);
        ExtCountedBars = IndicatorCounted();
        //---- check for possible errors
        if (ExtCountedBars<0) return(-1);
        //---- last counted bar will be recounted
        if (ExtCountedBars>0) ExtCountedBars--;
        //----
        switch(MA_Method)
        {
        case FT_SMA  : 
            sma();  
            break;

        case FT_EMA  : 
            ema();  
            break;

        case FT_SMMA : 
            smma(); 
            break;

        case FT_LWMA : 
            lwma(); 
            break;

        default: 
            throw Utils::CUtilsError(L"  ");
        }

        //---- done
        return(0);
    }
    //+------------------------------------------------------------------+
    //| Simple Moving Average                                            |
    //+------------------------------------------------------------------+
    void sma()
    {
        double sum=0;
        int    i,pos=Bars-ExtCountedBars-1;
        //---- initial accumulation
        if(pos<MA_Period) pos=MA_Period;
        for(i=1;i<MA_Period;i++,pos--)
            sum+=Close[pos];
        //---- main calculation loop
        while(pos>=0)
        {
            sum+=Close[pos];
            ExtMapBuffer[pos]=sum/MA_Period;
            sum-=Close[pos+MA_Period-1];
            pos--;
        }
        //---- zero initial bars
        if(ExtCountedBars<1)
            for(i=1;i<MA_Period;i++) ExtMapBuffer[Bars-i]=0;
    }
    //+------------------------------------------------------------------+
    //| Exponential Moving Average                                       |
    //+------------------------------------------------------------------+
    void ema()
    {
        double pr=2.0/(MA_Period+1);
        int    pos=Bars-2;
        if(ExtCountedBars>2) pos=Bars-ExtCountedBars-1;
        //---- main calculation loop
        while(pos>=0)
        {
            if(pos==Bars-2) ExtMapBuffer[pos+1]=Close[pos+1];
            ExtMapBuffer[pos]=Close[pos]*pr+ExtMapBuffer[pos+1]*(1-pr);
            pos--;
        }
    }
    //+------------------------------------------------------------------+
    //| Smoothed Moving Average                                          |
    //+------------------------------------------------------------------+
    void smma()
    {
        double sum=0;
        int    i,k,pos=Bars-ExtCountedBars+1;
        //---- main calculation loop
        pos=Bars-MA_Period;
        if(pos>Bars-ExtCountedBars) pos=Bars-ExtCountedBars;
        while(pos>=0)
        {
            if(pos==Bars-MA_Period)
            {
                //---- initial accumulation
                for(i=0,k=pos;i<MA_Period;i++,k++)
                {
                    sum+=Close[k];
                    //---- zero initial bars
                    ExtMapBuffer[k]=0;
                }
            }
            else sum=ExtMapBuffer[pos+1]*(MA_Period-1)+Close[pos];
            ExtMapBuffer[pos]=sum/MA_Period;
            pos--;
        }
    }
    //+------------------------------------------------------------------+
    //| Linear Weighted Moving Average                                   |
    //+------------------------------------------------------------------+
    void lwma()
    {
        double sum=0.0,lsum=0.0;
        double price;
        int    i,weight=0,pos=Bars-ExtCountedBars-1;
        //---- initial accumulation
        if(pos<MA_Period) pos=MA_Period;
        for(i=1;i<=MA_Period;i++,pos--)
        {
            price=Close[pos];
            sum+=price*i;
            lsum+=price;
            weight+=i;
        }
        //---- main calculation loop
        pos++;
        i=pos+MA_Period;
        while(pos>=0)
        {
            ExtMapBuffer[pos]=sum/weight;
            if(pos==0) break;
            pos--;
            i--;
            price=Close[pos];
            sum=sum-lsum+price*MA_Period;
            lsum-=Close[i];
            lsum+=price;
        }
        //---- zero initial bars
        if(ExtCountedBars<1)
            for(i=1;i<MA_Period;i++) ExtMapBuffer[Bars-i]=0;
    }
    //+------------------------------------------------------------------+

};





